/*  bsp.h
 *
 *  This include file contains all DMV152 board IO definitions.
 *
 *  COPYRIGHT (c) 1989-1999.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.rtems.com/license/LICENSE.
 *
 *  $Id: bsp.h,v 1.28 2006/03/17 10:11:36 ralf Exp $
 */

#ifndef _BSP_H
#define _BSP_H

#ifdef __cplusplus
extern "C" {
#endif

#include <bspopts.h>

#include <rtems.h>
#include <rtems/console.h>
#include <rtems/clockdrv.h>
#include <rtems/iosupp.h>
#include <rtems/vmeintr.h>
#include <rtems/zilog/z8530.h>
#include <rtems/zilog/z8536.h>

/*
 *  confdefs.h overrides for this BSP:
 *   - number of termios serial ports (defaults to 1)
 *   - Interrupt stack space is not minimum if defined.
 */

/* #define CONFIGURE_NUMBER_OF_TERMIOS_PORTS 2 */
#define CONFIGURE_INTERRUPT_STACK_MEMORY  (4 * 1024)

/*
 *  confdefs.h overrides for this BSP:
 *   - termios serial ports (defaults to 1)
 *   - Interrupt stack space is not minimum if defined.
 */

#define CONFIGURE_NUMBER_OF_TERMIOS_PORTS 2
/* #define CONFIGURE_INTERRUPT_STACK_MEMORY  (TBD * 1024) */

/*
 *  Simple spin delay in microsecond units for device drivers.
 *  This is very dependent on the clock speed of the target.
 */

#define rtems_bsp_delay( microseconds ) \
  { register uint32_t         _delay=(microseconds); \
    register uint32_t         _tmp=123; \
    asm volatile( "0: \
                     nbcd      %0 ; \
                     nbcd      %0 ; \
                     dbf       %1,0b" \
                  : "=d" (_tmp), "=d" (_delay) \
                  : "0"  (_tmp), "1"  (_delay) ); \
  }

/* macros */

#undef Z8x36_STATE0
#undef Z8x36_WRITE
#undef Z8x36_READ

#define Z8x36_STATE0 ( z8536 ) \
  { char *garbage; \
    (garbage) = *(VOL8(z8536+0x7)) \
  }

#define Z8x36_WRITE( z8536, reg, data ) \
   *(VOL8(z8536+0x7)) = (reg); \
   *(VOL8(z8536+0x7)) = (data)

#define Z8x36_READ( z8536, reg, data ) \
   *(VOL8(z8536+0x7)) = (reg); \
   (data) = *(VOL8(z8536+0x7))

/*
 *  ACC Register Addresses
 */

#define ACC_BASE    0x0D000000

#define ACC_STAT0   ((volatile uint8_t*) (ACC_BASE + 0x00))
#define ACC_STAT1   ((volatile uint8_t*) (ACC_BASE + 0x01))
#define ACC_GENCTL  ((volatile uint8_t*) (ACC_BASE + 0x02))
#define ACC_VINT    ((volatile uint8_t*) (ACC_BASE + 0x03))
#define ACC_VREQ    ((volatile uint8_t*) (ACC_BASE + 0x04))
#define ACC_VARB    ((volatile uint8_t*) (ACC_BASE + 0x05))
#define ACC_ID      ((volatile uint8_t*) (ACC_BASE + 0x06))
#define ACC_CTL2    ((volatile uint8_t*) (ACC_BASE + 0x07))
#define ACC_7IS     ((volatile uint8_t*) (ACC_BASE + 0x08))
#define ACC_LIS     ((volatile uint8_t*) (ACC_BASE + 0x09))
#define ACC_7IE     ((volatile uint8_t*) (ACC_BASE + 0x0A))
#define ACC_LIE     ((volatile uint8_t*) (ACC_BASE + 0x0B))
#define ACC_VIE     ((volatile uint8_t*) (ACC_BASE + 0x0C))
#define ACC_IC10    ((volatile uint8_t*) (ACC_BASE + 0x0D))
#define ACC_IC32    ((volatile uint8_t*) (ACC_BASE + 0x0E))
#define ACC_IC54    ((volatile uint8_t*) (ACC_BASE + 0x0F))

/* constants */

#define TIMER 0x0c000000
#define TIMER_VECTOR 0x4D

#define CONSOLE_CONTROL_A  0x0C800005
#define CONSOLE_DATA_A     0x0C800007
#define CONSOLE_CONTROL_B  0x0C800001
#define CONSOLE_DATA_B     0x0C800003

/* Structures */

   /* none */

/* miscellaneous stuff assumed to exist */

extern rtems_configuration_table BSP_Configuration;

extern m68k_isr_entry M68Kvec[];   /* vector table address */

/*
 *  Device Driver Table Entries
 */

/*
 * NOTE: Use the standard Console driver entry
 */

/*
 * NOTE: Use the standard Clock driver entry
 */

/* functions */

void bsp_cleanup( void );

m68k_isr_entry set_vector(
  rtems_isr_entry     handler,
  rtems_vector_number vector,
  int                 type
);

#ifdef __cplusplus
}
#endif

#endif
